/*****************************************************************************
 Copyright (c) 1993 UUNET Technologies
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. All advertising materials mentioning features or use of this software
    must display the following acknowledgement:
      This product includes software developed by UUNET Technologies
 4. The name UUNET Technologies may not be used to endorse or promote products  
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE UUNET Technologies ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE UUNET Technologies BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

*****************************************************************************/


#include <sys/types.h>
#include <netinet/in.h>
#include "muse.h"

#define Inline inline

static unsigned char *raw;
static mpacket space;


static Inline void rawTc(unsigned char *c)
{
	*c = *raw++;
}

static Inline void rawTs(unsigned char *s, int len)
{
	bcopy(raw, s, len);
	raw += len;
}

static Inline void rawTint32(int32 *v)
{
	*v = 0;
	bcopy(raw, v, 4);
	*v = ntohl(*v);
	raw += 4;
}

static Inline void rawTuint32(uint32 *v)
{
	*v = 0;
	bcopy(raw, v, 4);
	*v = ntohl(*v);
	raw += 4;
}

static Inline void rawTint16(int16 *v)
{
	*v = 0;
	bcopy(raw, v, 2);
	*v = ntohs(*v);
	raw += 2;
}

static Inline void sTraw(unsigned char *s, int len)
{
	bcopy(s, raw, len);
	raw += len;
}

static Inline void cTraw(unsigned char c)
{
	*raw++ = c;
}

static Inline void int32Traw(int32 v)
{
	v = htonl(v);
	bcopy(&v, raw, sizeof(int32));
	raw += 4;
}

static Inline void uint32Traw(uint32 v)
{
	v = htonl(v);
	bcopy(&v, raw, sizeof(uint32));
	raw += 4;
}

static Inline void int16Traw(int16 v)
{
	v = htons(v);
	bcopy(&v, raw, sizeof(int16));
	raw += 2;
}

void raw2mpacket(unsigned char *raw_in, int plen, mpacket *p, int *retslen,
  void **sig)
{
	raw = raw_in;

	rawTs(p->header.version, 4);
	rawTuint32(&p->header.sequence);
	*sig = raw;
	rawTs(p->header.rsa_sig, 64);
	rawTint16(&p->header.mesgid_off);
	rawTint16(&p->header.group_off);
	rawTc(&p->header.keyno);
	rawTc(&p->header.cmdfkr);
	rawTint32(&p->header.sec);
	rawTint32(&p->header.fsec);
	*retslen = plen - (raw - raw_in);
	rawTs(p->message, *retslen);
}

/* Returns size of packet packet */
void *mpacket2raw(mpacket *p, int slen, int *retlen, void **sig)
{
	bzero(space, sizeof(space));
	raw = (unsigned char *)&space;

	sTraw(p->header.version, 4);
	uint32Traw(p->header.sequence);
	*sig = raw;
	sTraw(p->header.rsa_sig, 64);
	int16Traw(p->header.mesgid_off);
	int16Traw(p->header.group_off);
	cTraw(p->header.keyno);
	cTraw(p->header.cmdfkr);
	int32Traw(p->header.sec);
	int32Traw(p->header.fsec);
	sTraw(p->message, slen);

	*retlen = raw - (unsigned char *)&space;
	return &space;
}
